# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     log
   Description :
   Author :       艾登科技 Asdil
   date：          2020/7/9
-------------------------------------------------
   Change Activity:
                   2020/7/9:
-------------------------------------------------
"""
__author__ = 'Asdil'
import logging


def simple_init(level='INFO', log_path=None):
    """add方法用于新建一个log

    Parameters
    ----------
    level: str
        日志级别
    log_path: str
        保存路径
    Returns
    ----------
    """
    logger = logging.getLogger()
    level_dict = {'DEBUG': logging.DEBUG, 'INFO': logging.INFO, 'WARNING': logging.WARNING, 'ERROR': logging.ERROR}
    logger.setLevel(level_dict[level])

    if log_path is not None:
        # create a file handler
        handler = logging.FileHandler(log_path)
        handler.setLevel(level)
        # create a logging format
        formatter = logging.Formatter('%(asctime)s | %(levelname)s | %(filename)s:%(funcName)s:%(lineno)s - %(message)s')
        handler.setFormatter(formatter)

    # add the handlers to the logger
    logger.addHandler(handler)
    return logger


def init(log_path, level='INFO'):
    """init方法用于新建一个log, 但不能在spark上面跑

    Parameters
    ----------
    log_path : str
        路径
    level : str
        日志等级
        
    Returns
    ----------
    """
    from loguru import logger
    logger.add(log_path, rotation="1 MB", enqueue=True, level=level)
    return logger





# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     tool
   Description :
   Author :        Asdil
   date：          2019/12/26
-------------------------------------------------
   Change Activity:
                   2019/12/26:
-------------------------------------------------
"""
__author__ = 'Asdil'
import os
import gzip
import time
import shutil
import subprocess
from tqdm import tqdm
from datetime import datetime
from inspect import signature


def path_join(path1, path2):
    """
    合并两个目录
    :param path1:  路径
    :param path2:  文件名
    :return:
    """
    assert isinstance(path1, str)
    assert isinstance(path2, str)
    if path1[-1] != os.sep:
        path1 += os.sep
    if path2[0] == os.sep:
        path2 = path2[1:]
    return path1 + path2


def get_files(path, extension=None, key=None):
    """
    获取目标目录文件
    :param path:      路径
    :param extension: 后缀
    :param key:       关键字
    :return:
    """
    if extension is not None:
        length = -len(extension)
        ret = [path_join(path, each) for each in os.listdir(
            path) if each[length:] == extension]
    elif key is not None:
        ret = [path_join(path, each)
               for each in os.listdir(path) if key in each]
    else:
        ret = [path_join(path, each) for each in os.listdir(path)]
    return ret


def get_name(path, extension=None, key=None):
    """
    获取目标目录下文件名
    :param path:      路径
    :param extension: 后缀
    :param key:       关键字
    :return:
    """
    if extension is not None:
        l = -len(extension)
        ret = [each for each in os.listdir(path) if each[l:] == extension]
    elif key is not None:
        ret = [each for each in os.listdir(path) if key in each]
    else:
        ret = [each for each in os.listdir(path)]
    return ret


def bar(data):
    """
    进度条
    :param data: 列表 字典 迭代器
    :return:
    """
    if isinstance(data, int):
        return tqdm(range(data))
    return tqdm(data)


def subprocess_check_call(cmd):
    """
    执行命令行命令
    :param cmd:  命令行命令
    :return:
    """
    subprocess.check_call(cmd, shell=True)


def subprocess_call(cmd):
    """
    执行命令行命令，不检查
    :param cmd:  命令行命令
    :return:
    """
    subprocess.call(cmd, shell=True)


def subprocess_popen(cmd):
    """
    执行命令获取返回值
    :param cmd:  命令行命令
    :return:
    """
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    out, err = p.communicate()
    return [each for each in out.decode('utf8').splitlines()]


def split_path(path):
    """
    拆分目录
    eg: '/tmp/tmp/a.txt'
        '/tmp/tmp', 'a.txt', 'a', 'txt'
    :param path: 路径
    :return:
    """
    assert isinstance(path, str)
    file_path, file_full_name = os.path.split(path)
    file_name, extension = os.path.splitext(file_full_name)
    return file_path, file_name, extension, file_full_name


def copy_file(srcfile, dstfile):
    """
    复制文件
    :param srcfile: 拷贝文件路径
    :param dstfile: 目标路径
    :return:
    """

    if not os.path.isfile(srcfile):
        print("%s not exist!" % srcfile)
        assert os.path.isfile(srcfile) is True
    else:
        _, _, _, name = split_path(srcfile)
        if dstfile[-len(name):] == name:
            fpath, fname = os.path.split(dstfile)  # 分离文件名和路径
        else:
            fpath = dstfile

        if not os.path.exists(fpath):
            os.makedirs(fpath)  # 创建路径

        dstfile = path_join(fpath, name)
        shutil.copyfile(srcfile, dstfile)  # 复制文件
        print("copy %s -> %s" % (srcfile, dstfile))


def cut_file(srcfile, dstfile):
    """
    剪切文件
    :param srcfile: 剪切文件路径
    :param dstfile: 目标路径
    :return:
    """
    if not os.path.isfile(srcfile):
        print("%s not exist!" % srcfile)
        assert os.path.isfile(srcfile) is True
    else:
        fpath, fname = os.path.split(dstfile)    # 分离文件名和路径
        if not os.path.exists(fpath):
            os.makedirs(fpath)                 # 创建路径
        shutil.move(srcfile, dstfile)          # 复制文件
        print("cut %s -> %s" % (srcfile, dstfile))


def re_write_txt(file_Path, key, newstr):
    """
    替换txt文件关键字
    :param file_Path: 文件路径
    :param key:       关键字
    :param newstr:    替换词
    :return:
    """
    content = ""
    with open(file_Path, "r", encoding="utf-8") as f:
        for line in f:
            if key in line:
                line = newstr + '\n'
            content += line
    # 写入
    with open(file_Path, "w", encoding="utf-8") as f:
        f.write(content)


def inter_set(l1, l2):
    """
    列表交集
    :param l1:
    :param l2:
    :return:
    """
    assert type(l1) in [list, set]
    assert type(l2) in [list, set]
    return list(set(l1).intersection(set(l2)))


def diff_set(l1, l2):
    """
    列表差集
    :param l1:
    :param l2:
    :return:
    """
    assert type(l1) in [list, set]
    assert type(l2) in [list, set]
    return list(set(l1).difference(set(l2)))


def union_set(l1, l2):
    """
    列表并集
    :param l1:
    :param l2:
    :return:
    """
    assert type(l1) in [list, set]
    assert type(l2) in [list, set]
    return list(set(l1).union(set(l2)))


def create_dir(path):
    """
    检查文件夹是否存在，如果存在则删除重新创建
    :param path:    文件夹路径
    :param type:    文件夹不存在是否报错  True报错， False不报错,并创建文件夹
    :return:
    """
    if not os.path.exists(path):
        os.makedirs(path)
        return True
    return False


def del_dir(path):
    """
    删除目录
    :param path:  路径
    :return:
    """
    shutil.rmtree(path)


def combin_dic(*args):
    """
    合并字典
    :param args: 两个字典或多个
    :return:
    """
    ret = {}
    if len(args) == 1:
        dicts = args[0]
        assert isinstance(dicts, list)  # 断言是个列表
        for _dict in dicts:
            ret = dict(ret, **_dict)
    else:
        for _dict in args:
            assert isinstance(_dict, dict)
        for _dict in args:
            ret = dict(ret, **_dict)
    return ret


def add_dic(dica, dicb):
    """
    字典累加
    :param dica:   字典a
    :param dicb:   字典b
    :return:       字典累加
    """
    dic = {}
    for key in dica:
        if dicb.get(key):
            dic[key] = dica[key] + dicb[key]
        else:
            dic[key] = dica[key]
    for key in dicb:
        if dica.get(key):
            pass
        else:
            dic[key] = dicb[key]
    return dic


def split_list(_list, slice):
    """
    拆分列表
    :param _list:  列表
    :param slice:  拆分块的大小
    :return:       拆分后的列表
    """
    return [_list[i:i + slice] for i in range(0, len(_list), slice)]


def zip_file(file_path, output=None, rename=None, typ=3):
    """
    压缩文件
    :param file_path:  文件绝对路径
    :param output:     是否输入到其它文件夹
    :return:           True, False
    """
    import zipfile
    # 拆分成文件路径，文件
    path, name, _, name_extension = split_path(file_path)
    if rename is None:
        rename = name

    if output is None:
        output = path
    azip = zipfile.ZipFile(path_join(output, rename + '.zip'), 'w')
    # 写入zip
    if typ == 1:
        azip.write(file_path, name_extension, compress_type=zipfile.ZIP_LZMA)

    elif typ == 2:
        azip.write(file_path, name_extension, compress_type=zipfile.ZIP_BZIP2)
    else:
        azip.write(
            file_path,
            name_extension,
            compress_type=zipfile.ZIP_DEFLATED)
    azip.close()
    print("{} -> {}".format(file_path, path_join(output, rename + '.zip')))


def unzip_file(file_path, output=None):
    """
    解压文件
    :param file_path:  zip文件完整路径
    :return:
    """
    import zipfile
    path, name, _, name_extension = split_path(file_path)
    azip = zipfile.ZipFile(file_path)
    if output is None:
        azip.extractall(path=output)
        output = path_join(path, name)
    else:
        azip.extractall(path=output)
        output = path_join(output, name)
    azip.close()
    print("{} ->> {}".format(file_path, output))


def zip_dir(file_dir, output=None, rename=None):
    """
    压缩文件夹
    :param file_dir:  文件夹路径
    :param output:    输出路径
    :param rename:    重命名
    :return:
    """
    if rename is None:
        tmp = file_dir.strip('/')
        dirs = tmp.strip('/').split('/')
        rename = dirs[-1]
    # 压缩文件夹
    if output is None:
        output = '/' + '/'.join(dirs[:-1])
        print(path_join(output, rename))
        shutil.make_archive(path_join(output, rename), 'zip', file_dir)
    else:
        shutil.make_archive(path_join(output, rename), 'zip', file_dir)
    print("{} -> {}".format(file_dir, path_join(output, rename) + '.zip'))


def unzip_dir(file_dir, output=None, rename=None):
    """
    解压文件夹
    :param file_dir:  解压文件夹
    :return:
    """
    path, name, _, _ = split_path(file_dir)
    if output is None:
        output = path
    if rename is None:
        rename = name
    output = path_join(output, rename)

    shutil.unpack_archive(file_dir, output)
    print('{} ->> {}'.format(file_dir, output))


def gzip_file(file_path, output=None, rename=None, del_file=False):
    """
    gzip文件
    :param file_path: 文件路径
    :param output:    输出路径
    :param rename:    重命名
    :param del_file:  是否删除源文件
    :return:
    """
    assert os.path.exists(file_path)
    path, name, _, name_extension = split_path(file_path)
    if rename is None:
        rename = name
    if output is None:
        output = path
    rename += '.gz'
    out_path = path_join(output, rename)
    with open(file_path, 'rb') as f_in:
        with gzip.open(out_path, 'wb') as f_out:
            shutil.copyfileobj(f_in, f_out)
    if del_file:
        os.remove(file_path)
    print('{} ->> {}'.format(file_path, out_path))


def gunzip_file(file_path, output=None, rename=None, del_file=False):
    """
    解压gz文件
    :param file_path: 文件路径
    :param output:    输出路径
    :param rename:    重命名
    :param del_file:  是否删除源文件
    :return:
    """
    assert os.path.exists(file_path)
    path, name, _, name_extension = split_path(file_path)
    if rename is None:
        rename = name
    if output is None:
        output = path
    if rename[-3:] == '.gz':
        rename = rename[:-3]
    out_path = path_join(output, rename)
    with gzip.open(file_path, 'rb') as f_in:
        data = f_in.read().decode('utf8')
        with open(out_path, 'w') as f_out:
            f_out.write(data)
    if del_file:
        os.remove(file_path)
    print('{} ->> {}'.format(file_path, out_path))


def until(y=None, m=None, d=None, H=None, M=None, S=None, logger=None):
    """
    定时任务
    :param y:  年
    :param m:  月
    :param d:  日
    :param H:  时
    :param M:  分
    :param S:  秒
    :param logger: 日志
    :return:
    """
    import time
    import datetime
    if y:
        y = int(y)
        m = int(m)
        d = int(d)
        H = int(H)
        M = int(M)
        S = int(S)
        try:
            startTime = datetime.datetime(y, m, d, H, M, S)
        except BaseException:
            if logger:
                logger.info('年月日时分秒输入错误')
            print('年月日时分秒输入错误')
            assert 1 == 2
        if startTime < datetime.datetime.now():
            logger.info('开始时间在当前时间之前')
            print('开始时间在当前时间之前')
            assert 2 == 3

        second = (startTime - datetime.datetime.now()).seconds
        minute = second // 60
        second = second % 60
        hour = minute // 60
        minute = minute % 60
        day = hour // 24
        hour = hour % 24

        print(f'将于{day}天{hour}小时{minute}分{second}秒 后运行')
        if logger:
            logger.info(f'将于{day}天{hour}小时{minute}分{second}秒 后运行')

        while datetime.datetime.now() < startTime:
            time.sleep(1)
        print('到达预定时间开始运行程序')
        if logger:
            logger.info('到达预定时间开始运行程序')
    else:
        if d or H or M or S:
            if H is None:
                H = 0
            if M is None:
                M = 0
            if S is None:
                S = 0
            seconds = 0
            time_dic = {'day': 86400,
                        'hour': 3600,
                        'min': 60}
            if d:
                seconds = (
                    time_dic['day'] *
                    int(d) +
                    time_dic['hour'] *
                    int(H) +
                    time_dic['min'] *
                    int(M) +
                    int(S))
                print(f'将于{d}天{H}小时{M}分{S}秒 后运行')
                if logger:
                    logger.info(f'将于{d}天{H}小时{M}分{S}秒 后运行')
            elif H:
                seconds = (
                    time_dic['hour'] *
                    int(H) +
                    time_dic['min'] *
                    int(M) +
                    int(S))
                print(f'将于{H}小时{M}分{S}秒 后运行')
                if logger:
                    logger.info(f'将于{H}小时{M}分{S}秒 后运行')
            elif M:
                seconds = (time_dic['min'] * int(M) + int(S))
                print(f'将于{M}分{S}秒 后运行')
                if logger:
                    logger.info(f'将于{M}分{S}秒 后运行')
            else:
                seconds = int(S)
                print(f'将于{S}秒 后运行')
                if logger:
                    logger.info(f'将于{S}秒 后运行')
            time.sleep(seconds)
            print('到达预定时间开始运行程序')
            if logger:
                logger.info('到达预定时间开始运行程序')
        else:
            print('错误！ 定时任务没有指定时间')
            if logger is not None:
                logger.info('错误！ 定时任务没有指定时间')
                assert 3 == 4


def get_process_id(name):
    """
    获取进程pid
    :param name:
    :return:
    """
    child = subprocess.Popen(["pgrep", "-f", name],
                             stdout=subprocess.PIPE, shell=False)
    response = child.communicate()[0]
    response = response.decode().strip().split('\n')
    if len(response) == 1 and len(response[0]) == 0:
        return []
    return response


def monitor_memery_cpu(pids, second=10, out_path=None, show=False):
    """
    pu使用率
    :param pids:
    :param second:
    :param out_path:
    :param show:
    :return:
    """
    import psutil
    proc = psutil.Process(int(pids))
    info = ['cpu rate\tmemory use']
    while True:
        try:
            cpu = psutil.cpu_percent()
            memory = proc.memory_info().rss / 1024 / 1024
        except BaseException:
            break
        if show:
            print(f'cpu 使用率: {cpu}%  memory 使用量 {round(memory, 2)}MB')
        info.append(f'{cpu}\t{memory}')
        time.sleep(second)
    if out_path is not None:
        with open(out_path, 'w') as f:
            f.write('\n'.join(info))


def read(path, sep='\n', encoding='utf-8'):
    """
    按行读数据
    :param path: 路径
    :param sep:  分隔符
    :param encoding: 编码
    :return:
    """
    with open(path, 'r', encoding=encoding) as f:
        return f.read().strip().split(sep)


def write(data, path, sep='\n', encoding='utf-8'):
    """write方法用于写数据

    Parameters
    ----------
    data : list
        列表数据
    path : str
        存储路径
    sep : str
        分隔符
    encoding : str
        编码种类

    Returns
    ----------
    """
    with open(path, 'w', encoding=encoding) as f:
        f.write(sep.join(data))


def merge_commelement_list(lsts):
    """
    把公共元素的列表合并，返回合并后的结果list
    :param lsts:
    :return:
    """
    sets = [set(lst) for lst in lsts if lst]
    merged = 1
    while merged:
        merged = 0
        results = []
        while sets:
            common, rest = sets[0], sets[1:]
            sets = []
            for x in rest:
                if x.isdisjoint(common):
                    sets.append(x)
                else:
                    merged = 1
                    common |= x
            results.append(common)
        sets = results
    return sets


def runtime(func):
    """
    运行时间的装饰器
    :param : python function
    :return:
    """
    def wrapper(*args, **kwargs):
        start_now = datetime.now()
        start_time = time.time()
        ret = func(*args, **kwargs)
        end_time = time.time()
        end_now = datetime.now()
        print(f'time时间:{end_time-start_time}')
        print(
            f'datetime起始时间:{start_now} 结束时间:{end_now}, 一共用时{end_now-start_now}')
        return ret
    return wrapper


def typeassert(*ty_args, **ty_kwargs):
    """typeassert方法用于强制确认输入格式

    @typeassert(int, b=str)
    f(a, b)

    Parameters
    ----------
    param : str

    Returns
    ----------
    """
    from inspect import signature
    from functools import wraps

    def decorate(func):
        # If in optimized mode, disable type checking
        if not __debug__:
            return func

        # Map function argument names to supplied types
        sig = signature(func)
        bound_types = sig.bind_partial(*ty_args, **ty_kwargs).arguments

        @wraps(func)
        def wrapper(*args, **kwargs):
            bound_values = sig.bind(*args, **kwargs)
            # Enforce type assertions across supplied arguments
            for name, value in bound_values.arguments.items():
                if name in bound_types:
                    if not isinstance(value, bound_types[name]):
                        raise TypeError(
                            'Argument {} must be {}'.format(
                                name, bound_types[name]))
            return func(*args, **kwargs)
        return wrapper
    return decorate


def install(package):
    """install方法用于安装包

    Parameters
    ----------
    package : str
        包名
    Returns
    ----------
    """
    import pip
    if hasattr(pip, 'main'):
        pip.main(['install', package])
    else:
        pip._internal.main(['install', package])


def flatten(data):
    """flatten方法用于平铺list

    Parameters
    ----------
    data : list
        列表

    Returns
    ----------
    """
    import itertools
    return list(itertools.chain.from_iterable(data))


def read_json(path):
    """read_json方法用于读取json文件

    Parameters
    ----------
    path : str
        json文件路径

    Returns
    ----------
    """
    import json
    with open(path, 'r', encoding='utf-8') as f:
        data = json.loads(f.read())
    return data


def write_json(data, path, sort=False):
    """write_json方法用于写json到文件中

    Parameters
    ----------
    data : str
        字典文件
    path : str
        保存路径
    sort : bool
        是否排序
    Returns
    ----------
    """
    import json
    with open(path, "w", encoding='UTF-8') as f:
        f.write(json.dumps(data, indent=4, ensure_ascii=False, sort_keys=sort))


def show_memory(variable, unit='KB'):
    """show_memory方法用于查看变量占用内存多少

    Parameters
    ----------
    variable : str
        变量名称
    unit : str
        单位 B, KB, MB, GB
    Returns
    ----------
    """
    from sys import getsizeof
    scale = {'B': 1, 'KB': 1024, 'MB': 1048576, 'GB': 1073741824}[unit]
    memory = eval("getsizeof({})".format(variable)) // scale
    print(f'{variable}: {memory} {unit}')
    return memory


def lists_compare(l1, l2):
    """lists_compare方法用于比较两个列表差异性

    Parameters
    ----------
    l1 : list
        列表1
    l2 : list
        列表2

    Returns
    ----------
    """
    dif12 = diff_set(l1, l2)
    dif12_num = len(dif12)
    dif21 = diff_set(l2, l1)
    dif21_num = len(dif21)
    inter = inter_set(l1, l2)
    inter_num = len(inter)
    union = union_set(l1, l2)
    union_num = len(union)
    print(f'l1 与 l2 差集个数{dif12_num}')
    print(f'l2 与 l1 差集个数{dif21_num}')
    print(f'l2 与 l1 交集个数{inter_num}')
    print(f'l2 与 l1 并集个数{union_num}')
    return dif12_num, dif21_num, inter_num, union_num, dif12, dif21, inter, union_num


def iou(l1, l2):
    """iou方法用于计算交集/并集

    Parameters
    ----------
    l1 : list
        列表1
    l2 ： list
        列表2

    Returns
    ----------
    """
    inter = inter_set(l1, l2)
    union = union_set(l1, l2)
    if len(union) == 0:
        return 0
    return len(inter)/len(union)


def try_except(func):
    """try_except方法用于try except捕获异常

    Parameters
    ----------
    func : function
        函数
    detail ： bool
        是否打印细节
    Returns
    ----------
    """
    # import traceback
    def wrapper(*args, **keyargs):
        try:
            return func(*args, **keyargs)
        except Exception as e:
            print(f'函数 {func.__name__} 执行错误!')
            print(e)
            # traceback.print_exc()
            return None
    return wrapper


def list_to_dict(_list):
    """list_to_dict方法用于列表转化为字典

    Parameters
    ----------
    _list : list
        列表

    Returns
    ----------
    """
    from collections import defaultdict
    check = [item[0] for item in _list]  # 查看首元素是否有重复
    if len(check) == len(set(check)):
        _dict = {}
        for item in _list:
            key = item[0]
            item = item[1:]
            item = item[0] if len(item) == 1 else item
            _dict[key] = item
    else:
        _dict = defaultdict(list)
        for item in _list:
            key = item[0]
            item = item[1:]
            _dict[key].extend(item)
    return _dict


def dict_to_list(_dict):
    """dict_to_list方法用于字典转列表

    Parameters
    ----------
    _dict : dict
        字典

    Returns
    ----------
    """
    ret = []
    for key in _dict:
        data = _dict[key]
        if type(data) is list:
            ret.append([key, *data])
        else:
            ret.append([key, data])
    return ret


def write_yaml(data, path, encoding='UTF-8'):
    """write_yaml方法用于写yaml配置文件

    Parameters
    ----------
    data : dict
        配置字典
    path : str
        保存路径
    encoding : str
        编码规则

    Returns
    ----------
    """
    import yaml
    with open(path, 'w', encoding=encoding) as yaml_file:
        yaml.safe_dump(data, yaml_file)


def load_yaml(path, encoding='UTF-8'):
    """read_yaml方法用于读取yaml文件

    Parameters
    ----------
    path : str
        yaml文件路径
    encoding : str
        编码规则

    Returns
    ----------
    """
    import yaml
    with open(path, 'r', encoding=encoding) as f:
        config = yaml.load(f.read())
    return config





# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     deep
   Description :
   Author :        Asdil
   date：          2020/4/23
-------------------------------------------------
   Change Activity:
                   2020/4/23:
-------------------------------------------------
"""
__author__ = 'Asdil'


def example_pytorch():
    """get_simple_net方法用于显示简单的prtorch网络结构

    Parameters
    ----------

    Returns
    ----------
    """
    print('import torch')
    print('from torch import nn')
    print('from torch.nn import init')
    print('import torch.optim as optim')
    print('import torch.nn.functional as F')
    print('from torchvision import transforms')
    print('from torch.utils.data import Dataset')
    print('from torch.utils.data import DataLoader')
    print("device = torch.device('cuda' if torch.cuda.is_available else 'cpu')")
    print('')
    print('# 加载数据')
    print('class myDataset(Dataset):')
    print('    def __init__(self, feature, label, transform=None):')
    print('        self.feature = feature')
    print('        self.label = label')
    print('    def __len__(self):')
    print('        return len(self.feature)')
    print('    def __getitem__(self, idx):')
    print('        data = self.feature[idx,:]')
    print('        label = self.label[i]')
    print('        if self.transform:')
    print('            data = self.transform(data)')
    print('        return data, label')
    print('train_iter = DataLoader(dataset=, batch_size=, shuffle=True, num_workers=8)')
    print('')
    print('# 初始化权重')
    print('def _weight_init(m):')
    print('    if isinstance(m, nn.Linear):')
    print('        nn.init.xavier_uniform_(m.weight)')
    print('        nn.init.constant_(m.bias, 0)')
    print('    elif isinstance(m, nn.Conv2d):')
    print('        nn.init.xavier_uniform_(m.weight)')
    print('    elif isinstance(m, nn.BatchNorm1d):')
    print('        nn.init.constant_(m.weight, 1)')
    print('        nn.init.constant_(m.bias, 0)')
    print('')
    print('# 加载网络')
    print('class Net(nn.Module):')
    print('    def __init__(self):')
    print('        super(Net, self).__init__()')
    print('        self.apply(_weight_init) # 初始化权重')
    print('        do something...')
    print('    #正向传播 ')
    print('    def forward(self, x):')
    print('        do something...')
    print('        return x')
    print('')

    print('loss = nn.MSELoss()')
    print('optimizer = optim.SGD(net.parameters(), lr=0.03)')
    print('for epoch in range(1,epochs + 1):')
    print('    for X, y in data_iter:')
    print('        output = net(X)')
    print('        l = loss(output, y.view(-1, 1))')
    print('        optimizer.zero_grad() # 梯度清零，等价于net.zero_grad()')
    print('        l.backward()')
    print('        optimizer.step()')
    print("""    print('epoch %d, loss: %f' % (epoch, l.item()))""")


# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     database
   Description :
   Author :        Asdil
   date：          2020/8/24
-------------------------------------------------
   Change Activity:
                   2020/8/28:
-------------------------------------------------
"""
__author__ = 'Asdil'
import psycopg2
from psycopg2 import pool
from DBUtils.PersistentDB import PersistentDB
import pymysql


class Psycopg:
    """
    Psycopg类用于链接Psycopg
    """

    def __init__(self, host, port, user, password, database, minconn=2, maxconn=10):
        """__init__(self):方法用于

        Parameters
        ----------
        host : str
            host地址
        port : int
            端口
        user : str
            用户名
        password : str
            密码
        database : str
            数据库名称
        minconn : int
            最小连接数
        maxconn : int
            最大连接数

        Returns
        ----------
        """
        try:
            self.connectPool = pool.ThreadedConnectionPool(minconn, maxconn, host=host, port=port,
                                                           user=user, password=password,
                                                           database=database)

        except (Exception, psycopg2.DatabaseError) as error:
            print("Error while connecting to PostgreSQL", error)

    def get_connect(self):
        """getConnect方法用于生成新连接"""
        conn = self.connectPool.getconn()
        cursor = conn.cursor()
        return conn, cursor

    def close_connect(self, conn, cursor):
        """closeConnect方法用于关闭连接

        Parameters
        ----------
        cursor : object
            游标
        conn : object
            连接
        Returns
        ----------
        """
        cursor.close()
        self.connectPool.putconn(conn)

    def close_all(self):
        """close_all方法用于断开连接

        Parameters
        ----------
        param : str

        Returns
        ----------
        """
        self.connectPool.closeall()

    def select_one(self, sql, args=None):
        """select_one方法用于查找一条数据

        Parameters
        ----------
        sql : str
            查询语句
        args : list or None
            参数
        Returns
        ----------
        """
        conn, cursor = self.get_connect()
        if args:
            cursor.execute(sql, args)
        else:
            cursor.execute(sql)
        result = cursor.fetchone()
        self.close_connect(conn, cursor)
        return result

    def select_all(self, sql, args):
        """select_all方法用于查找所有记录

        Parameters
        ----------
        sql : str
            sql语句
        args : list or None
            参数
        Returns
        ----------
        """
        conn, cursor = self.get_connect()
        if args:
            cursor.execute(sql, args)
        else:
            cursor.execute(sql)
        result = cursor.fetchall()
        self.close_connect(conn, cursor)
        return result

    def excute(self, sql, value=None):
        """excute方法用于增删查改

        Parameters
        ----------
        sql : str
            sql 语句
        value : list or None
            值列表
        Returns
        ----------
        """
        conn, cursor = self.get_connect()
        try:
            res = cursor.execute(sql, value)
            conn.commit(conn, cursor)
            self.close_connect(conn, cursor)
            return res
        except Exception as e:
            conn.rollock()
            raise e


class Mysql:
    """
    Mysql类用于mysql连接池操作
    """
    def __init__(self, host, port, user, password, database):
        """__init__(self):方法用于

        Parameters
        ----------
        host : str
            host地址
        port : int
            端口
        user : str
            用户名
        password : str
            密码
        database : str
            数据库名称

        Returns
        ----------
        """
        self.pool = PersistentDB(
            creator=pymysql,  # 使用连接数据库模块
            maxusage=None,  # 连接超时时间
            setsession=[],  # 开始会话前执行的操作
            ping=0,  # ping服务器端查看是否可用
            closeable=False,  # 实际上被忽略，供下次使用，再线程关闭时，才会自动关闭链接。如果为True时， conn.close()则关闭链接，
            # 那么再次调用pool.connection时就会报错，因为已经真的关闭了连接（pool.steady_connection()可以获取一个新的链接
            threadlocal=None,  # 本线程独享值得对象，用于保存链接对象，如果链接对象被重置
            host=host,
            port=port,
            user=user,
            password=password,
            database=database,
            charset='utf8')

    def select_one(self, sql, args=None):
        """select_one方法用于查询一条数据

        Parameters
        ----------
        sql : str
            sql 语句
        args : list or tuple
            参数
        Returns
        ----------
        """
        conn = self.pool.connection(shareable=False)
        cursor = conn.cursor()
        cursor.execute(sql, args)
        result = cursor.fetchone()
        cursor.close()
        conn.close()
        return result

    def select_all(self, sql, args=None):
        """select_all方法用于查询所有数据

        Parameters
        ----------
        sql : str
            sql 语句
        args : list or tuple or None
            参数
        Returns
        ----------
        """
        if not args:
            args = tuple(args)
        conn = self.pool.connection(shareable=False)
        cursor = conn.cursor()
        cursor.execute(sql, args)
        result = cursor.fetchall()
        conn.close()
        return result

    def excute(self, sql, args=None):
        """excute方法用于增删查改

        Parameters
        ----------
        param : str

        Returns
        ----------
        """
        if not args:
            args = tuple(args)
        conn = self.pool.connection(shareable=False)
        cursor = conn.cursor()
        cursor.execute(sql, args)
        conn.commit()
        conn.close()


def optimize_expression(sql, params):
    """optimize_expression方法用于优化sql表达式
    使用方法:
        sql = "select * from table1 where age=%s and name in (%s)"
        params = [10, ['Tom', 'Jim']]
        sql, params = optimize_expression(sql, params)
    Parameters
    ----------
    sql : str
        sql语句
    params ： 参数列表

    Returns
    ----------
    """
    new_params = []
    codes = []
    for param in params:
        if isinstance(param, list):
            new_params.extend(list(map(str, param)))
            codes.append(','.join(['%s']*len(param)))
        else:
            new_params.append(param)
            codes.append('%s')
    sql = sql % tuple(codes)
    return sql, new_params





# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     help
   Description :
   Author :        艾登科技 Asdil
   date：          2020/8/31
-------------------------------------------------
   Change Activity:
                   2020/8/31:
-------------------------------------------------
"""
__author__ = 'Asdil'


def help_argparse():
    """help_argparse方法用于参数示例

    Parameters
    ----------
    param : str
        
    Returns
    ----------
    """
    print("import argparse")
    print("if __name__ == '__main__':")
    print("    args_parser = argparse.ArgumentParser(description='参数配置')")
    print("    args_parser.add_argument('--p', type=int, default=10, help='参数')")
    print("    args = args_parser.parse_args()")


def help_parallel():
    """help_parallel方法用于并行计算示例

    Parameters
    ----------
    param : str

    Returns
    ----------
    """
    print('from aftool import jobs')
    print('')
    print('def func():')
    print('    return 0')
    print('')
    print('args = []')
    print('mmp_df = jobs.memmap(mmp_df)')
    print('ret = jobs.parallel(args, func, njobs=8, backend=1)')


def help_ray():
    """ray方法用于ray是示例

    Parameters
    ----------
    param : str

    Returns
    ----------
    """
    print('import ray')
    print('ray.init(num_cpus=8)')
    print('args = []')
    print('')
    print('@ray.remote')
    print('def func(args):')
    print('    return 0')
    print('')
    print('df_id = ray.put(df) # 共享内存')
    print('ret = [f.remote() for i in args]')


def help_log():
    """help_log方法用于

    Parameters
    ----------
    param : str

    Returns
    ----------
    """

    print("logger = simple_init(level='INFO', log_path=None)")
    print("logger.info('第一种方法')")

    print("logger = init(log_path, level='INFO')")
    print("logger.info('第二种方法')")
    return 0


def help_pytorch():
    """help_pytorch方法用于显示简单的prtorch网络结构

    Parameters
    ----------
    param : str

    Returns
    ----------
    """
    print('import torch')
    print('from torch import nn')
    print('from torch.nn import init')
    print('import torch.optim as optim')
    print('import torch.nn.functional as F')
    print('from torchvision import transforms')
    print('from torch.utils.data import Dataset')
    print('from torch.utils.data import DataLoader')
    print("device = torch.device('cuda' if torch.cuda.is_available else 'cpu')")
    print('')
    print('# 加载数据')
    print('class myDataset(Dataset):')
    print('    def __init__(self, feature, label, transform=None):')
    print('        self.feature = feature')
    print('        self.label = label')
    print('    def __len__(self):')
    print('        return len(self.feature)')
    print('    def __getitem__(self, idx):')
    print('        data = self.feature[idx,:]')
    print('        label = self.label[i]')
    print('        if self.transform:')
    print('            data = self.transform(data)')
    print('        return data, label')
    print('train_iter = DataLoader(dataset=, batch_size=, shuffle=True, num_workers=8)')
    print('')
    print('# 初始化权重')
    print('def _weight_init(m):')
    print('    if isinstance(m, nn.Linear):')
    print('        nn.init.xavier_uniform_(m.weight)')
    print('        nn.init.constant_(m.bias, 0)')
    print('    elif isinstance(m, nn.Conv2d):')
    print('        nn.init.xavier_uniform_(m.weight)')
    print('    elif isinstance(m, nn.BatchNorm1d):')
    print('        nn.init.constant_(m.weight, 1)')
    print('        nn.init.constant_(m.bias, 0)')
    print('')
    print('# 加载网络')
    print('class Net(nn.Module):')
    print('    def __init__(self):')
    print('        super(Net, self).__init__()')
    print('        self.apply(_weight_init) # 初始化权重')
    print('        do something...')
    print('    #正向传播 ')
    print('    def forward(self, x):')
    print('        do something...')
    print('        return x')
    print('')

    print('loss = nn.MSELoss()')
    print('optimizer = optim.SGD(net.parameters(), lr=0.03)')
    print('for epoch in range(1,epochs + 1):')
    print('    for X, y in data_iter:')
    print('        output = net(X)')
    print('        l = loss(output, y.view(-1, 1))')
    print('        optimizer.zero_grad() # 梯度清零，等价于net.zero_grad()')
    print('        l.backward()')
    print('        optimizer.step()')
    print("""    print('epoch %d, loss: %f' % (epoch, l.item()))""")


def help_abspath():
    """help_path方法用于绝对路径

    Parameters
    ----------
    param : str

    Returns
    ----------
    """
    print("import os")
    print("abs_path = os.path.dirname(os.path.abspath(__file__))")
    print("father_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))")


def help_fastapi():
    """help_fastapi方法用于fastapi示例

    Parameters
    ----------
    param : str

    Returns
    ----------
    """
    print("from fastapi import FastAPI")
    print("from pydantic import BaseModel")
    print("app = FastAPI()")
    print("class Args(BaseModel):")
    print("    data_str: str")
    print("    data_int: int")
    print("    data_list: list")
    print("")
    print("""@app.post("/test_post")""")
    print("async def postEchoApi(args:Args):")
    print("    dict_args = args.dict() # 也可以转化为字典")
    print("""    return {"str data":args.data_str,""")
    print("""    'int data': args.data_int,""")
    print("""    'list data':args.data_list,""")
    print("""    'args 数据类型': str(type(args))}""")
    print("")
    print("if __name__ == '__main__':")
    print("    import uvicorn")
    print("""    uvicorn.run(app, host="127.0.0.1", port=8000)""")
    return 0



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     fo
   Description :
   Author :        Asdil
   date：          2019/12/26
-------------------------------------------------
   Change Activity:
                   2019/12/26:
-------------------------------------------------
"""
__author__ = 'Asdil'
# 文件操作
import os
import shutil
from aftool import tool


def is_dir(path):
    """
    是否是文件夹
    :param path:  路径
    :return:
    """
    return os.path.isdir(path)


def is_file(path):
    """
    是否是文件夹
    :param path:  路径
    :return:
    """
    return os.path.isfile(path)


def is_exist(path):
    """
    是否存在该文件
    :param path:  路径
    :return:
    """
    return os.path.exists(path)


def del_file(path):
    """
    删除文件
    :param path:  路径
    :return:
    """
    flag = True
    try:
        os.remove(path)
    except BaseException:
        flag = False
    return flag


def del_dir(path):
    """
    删除文件夹
    :param path:  路径
    :return:
    """
    flag = True
    try:
        shutil.rmtree(path)
    except BaseException:
        flag = False
    return flag


def del_all_files(path, key=None):
    """
    删除目录下所有文件
    :param path:
    :param key:
    :return:
    """
    if key is None:
        files = tool.get_files(path)
    else:
        files = tool.get_files(path, key=key)
    for _file in files:
        if is_file(_file):
            del_file(_file)
        else:
            del_dir(_file)
    return True


def copy_all_files(srcfile, dstfile, key=None, isreplace=False):
    """
    拷贝目录下所有文件
    :param srcfile:
    :param dstfile:
    :param key:
    :param isreplace:
    :return:
    """
    if key is None:
        files = tool.get_files(srcfile)
    else:
        files = tool.get_files(srcfile, key=key)
    if isreplace:
        for _file in files:
            if is_file(_file):
                _, _, _, name = tool.split_path(_file)
                if is_exist(_file):
                    del_file(tool.path_join(dstfile, name))
                tool.copy_file(_file, dstfile)
            else:
                _, _, _, name = tool.split_path(_file)
                if is_exist(_file):
                    del_dir(tool.path_join(dstfile, name))
                shutil.copytree(_file, dstfile + f'/{name}')
    else:
        for _file in files:
            if is_file(_file):
                tool.copy_file(_file, dstfile)
            else:
                _, _, _, name = tool.split_path(_file)
                shutil.copytree(_file, dstfile + f'/{name}')


def cut_all_files(srcfile, dstfile, key=None, isreplace=False):
    """
    拷贝目录下所有文件
    :param srcfile:
    :param dstfile:
    :param key:
    :param isreplace:
    :return:
    """
    if key is None:
        files = tool.get_files(srcfile)
    else:
        files = tool.get_files(srcfile, key=key)
    if isreplace:
        for _file in files:
            if is_file(_file):
                _, _, _, name = tool.split_path(_file)
                if is_exist(_file):
                    del_file(tool.path_join(dstfile, name))
                tool.cut_file(_file, dstfile)
            else:
                _, _, _, name = tool.split_path(_file)
                if is_exist(_file):
                    del_dir(tool.path_join(dstfile, name))
                shutil.move(_file, dstfile + f'/{name}')
                print(f'copy {_file} -> dstfile/{name}')
    else:
        for _file in files:
            if is_file(_file):
                tool.cutFile(_file, dstfile)
            else:
                _, _, _, name = tool.splitPath(_file)
                shutil.move(_file, dstfile + f'/{name}')
                print(f'copy {_file} -> dstfile/{name}')


def cut_all_files(srcfile, dstfile, key=None, isreplace=False):
    """
    拷贝目录下所有文件
    :param srcfile:
    :param dstfile:
    :param key:
    :param isreplace:
    :return:
    """
    if key is None:
        files = tool.get_files(srcfile)
    else:
        files = tool.get_files(srcfile, key=key)
    if isreplace:
        for _file in files:
            if is_file(_file):
                _, _, _, name = tool.split_path(_file)
                if is_exist(_file):
                    del_file(tool.path_join(dstfile, name))
                tool.cut_file(_file, dstfile)
            else:
                _, _, _, name = tool.split_path(_file)
                if is_exist(_file):
                    del_dir(tool.path_join(dstfile, name))
                shutil.move(_file, dstfile + f'/{name}')
                print(f'cut {_file} -> dstfile/{name}')
    else:
        for _file in files:
            if is_file(_file):
                tool.cut_file(_file, dstfile)
            else:
                _, _, _, name = tool.split_path(_file)
                shutil.move(_file, dstfile + f'/{name}')
                print(f'cut {_file} -> dstfile/{name}')



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     datetime
   Description :
   Author :        Asdil
   date：          2020/7/2
-------------------------------------------------
   Change Activity:
                   2020/7/2:
-------------------------------------------------
"""
__author__ = 'Asdil'
import time
from datetime import datetime
from dateutil.parser import parse


def str_to_datetime(date):
    """str_to_datetime方法用于字符串转日期

    Parameters
    ----------
    date : str
        日期字符串：
        eg:
        2018-10-21
        20181021
        2018/10/21
        10-21 # 如果没有年份默认今年
        10/21
    Returns
    ----------
    datetime
    """
    return parse(date)


def datetime_to_str(date, milliseconds=False):
    """datetime_to_str方法用于日期边字符串

    Parameters
    ----------
    date : datetime
        日期
    milliseconds: bool
        是否保留毫秒
    Returns
    ----------
    """
    if milliseconds:
        return date.isoformat(sep=' ', timespec='milliseconds')
    else:
        return date.replace(microsecond=0).isoformat(' ')


def timestamp_to_datetime(timestamp):
    """timestamp_to_str方法用于时间戳转日期字符串

    Parameters
    ----------
    timeStamp : float
        时间戳
    Returns
    ----------
    """
    return datetime.fromtimestamp(timestamp)


def datetime_to_timestamp(date):
    """datetime_to_timestamp方法用于日期转时间戳

    Parameters
    ----------
    date : datetime
        日期
    Returns
    ----------
    """
    return datetime.timestamp(date)


def timestamp_to_str(timestamp, milliseconds=False):
    """timestamp_to_str方法用于时间戳转日期字符串

    Parameters
    ----------
    timestamp : float
        时间戳
    milliseconds : bool
        保留毫秒
    Returns
    ----------
    """
    date = timestamp_to_datetime(timestamp)
    return datetime_to_str(date, milliseconds)


def now_str(format_type=2):
    """now_str方法用于返回当前时间,字符串类型

    Parameters
    ----------
    format_type : int
        显示类型
        0: 年月日时分秒
        1: 年-月-日 时:分:秒
        2: 年月日时分
        3: 年-月-日
    Returns
    ----------
    """
    if format_type == 0:
        return time.strftime('%Y%m%d%H%M%S', time.localtime(time.time()))
    elif format_type == 1:
        return time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
    elif format_type == 2:
        return time.strftime('%Y%m%d%H%M', time.localtime(time.time()))
    else:
        return time.strftime('%Y-%m-%d', time.localtime(time.time()))



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     geno
   Description :
   Author :        Asdil
   date：          2020/2/6
-------------------------------------------------
   Change Activity:
                   2020/2/6:
-------------------------------------------------
"""
__author__ = 'Asdil'
import pandas as pd
import numpy as np
import gzip


# 识别过滤的行数
def skipRow(path, key):
    skip = 0
    with open(path, 'r') as f:
        for line in f:
            if key == line[:len(key)]:
                return skip
            skip += 1
    return skip


# 读取call文件
def readCall(path, key='probeset_id'):
    df = pd.read_table(path, skiprows=skipRow(path, key), low_memory=False)
    return df


# 读取vcf
def readVcf(path, key='#CHROM'):
    df = pd.read_table(path, skiprows=skipRow(path, key), low_memory=False)
    return df


def skipRowGZ(path):
    skiprows = 0
    with gzip.open(path, 'rb') as f:
        for line in f:
            line = line.decode('utf-8')
            if line[0] != '#':
                break
            skiprows += 1
    skiprows -= 1
    return skiprows


# 读取vcf
def readVcfGZ(path):
    df = pd.read_table(path, skiprows=skipRowGZ(path), low_memory=False)
    return df


# 计算厘摩
class geneticMap(object):
    """keeps track of genetic Map locations and returns closest genetic map
    location given a snp location. """

    def __init__(self, file):
        """Builds conversion object for mapping base-pair position to genetic map position.
        Parameters:
        - `file` - tab delimited file with three columns: position [bp], combined rate [cM/Mb], Genetic Map [cM].
                   Example files for human genome release 36 can be found at:
                      ftp://ftp.hapmap.org/hapmap/recombination/2008-03_rel22_B36/rates/
                   and for relase 37 at:
                      ftp://ftp.hapmap.org/hapmap/recombination/2011-01_phaseII_B37/genetic_map_HapMapII_GRCh37.tar.gz
        """
        fp = open(file)
        # re-code the column name
        self.m = np.asarray([np.asarray(l.split())[[1, 3]]
                             for l in fp.readlines()[1:]], np.float)

    def pos2gm(self, pos):
        """Converts position in bp to position in centiMorgans"""
        m = self.m
        pos = np.asarray(pos)
        results = np.empty(pos.shape, dtype=np.float)
        # 查询pos元素在m[:,0]这个序列中得位置
        # Find probable locations in map 查询位点的遗传位置
        i = m[:, 0].searchsorted(pos)
        # Correct those that are beyond the end of the genetic map
        i[i == len(m)] = len(m) - 1
        # Fill results with those that match exactly
        idx = (m[i, 0] == pos) | (i == len(m) - 1) | (i == 0)
        results[idx] = m[i[idx], 1]

        idx = np.logical_not(idx)
        # Fill those that require interpolation
        results[idx] = (m[i[idx], 1] - m[i[idx] - 1, 1]) / (m[i[idx], 0] - m[i[idx] - 1, 0]) * \
                       (pos[idx] - m[i[idx] - 1, 0]) + m[i[idx] - 1, 1]
        return results


def hp():
    print(
        '计算厘摩: model = geneticMap("分染色体genomap文件")   pos=[1, 100] model.pos2gm(pos)')



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     encryption
   Description :
   Author :       艾登科技 Asdil
   date：          2020/7/10
-------------------------------------------------
   Change Activity:
                   2020/7/10:
-------------------------------------------------
"""
__author__ = 'Asdil'
import base64
import hashlib
from Crypto import Random
from Crypto.Cipher import AES


class AESCipher(object):

    def __init__(self, key):
        self.bs = AES.block_size
        self.key = hashlib.sha256(key.encode()).digest()

    def encrypt(self, raw):
        raw = self._pad(raw)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return base64.b64encode(iv + cipher.encrypt(raw.encode()))

    def decrypt(self, enc):
        enc = base64.b64decode(enc)
        iv = enc[:AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return self._unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')

    def _pad(self, s):
        return s + (self.bs - len(s) % self.bs) * chr(self.bs - len(s) % self.bs)

    @staticmethod
    def _unpad(s):
        return s[:-ord(s[len(s)-1:])]


def base_encrypt(data):
    """base_encrypt方法用于base64转码

    Parameters
    ----------
    data : str
        需要转码的字符串
    Returns
    ----------
    """
    data = bytes(data, encoding="utf8")
    encrypt_data = base64.b64encode(data)
    encrypt_data = str(encrypt_data, encoding="utf8")
    return encrypt_data


def base_decrypt(data):
    """base_decrypt方法用于

    Parameters
    ----------
    data : str
        转码后字段

    Returns
    ----------
    """
    data = bytes(data, encoding="utf8")
    decrypt_data = base64.b64decode(data)
    decrypt_data = str(decrypt_data, encoding="utf8")
    return decrypt_data




# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     joblib
   Description :
   Author :        Asdil
   date：          2020/3/3
-------------------------------------------------
   Change Activity:
                   2020/3/3:
-------------------------------------------------
"""
__author__ = 'Asdil'
import joblib
from aftool import tool
from aftool import fo
import tempfile
from tqdm import tqdm


def dump(py_object, save_path, compress=3):
    """dump方法用于持久化python数据

    Parameters
    ----------
    py_object : object
        python 变量
    save_path : str
        存储地址
    compress : int
        压缩率默认3, 数值越高压缩比越大

    Returns
    ----------
    """
    joblib.dump(py_object, save_path, compress=compress)
    print(f'{save_path} 保存成功！')


def load(file_path):
    """load方法用于读取持久化文件

    Parameters
    ----------
    file_path : str
        文件路径

    Returns
    ----------
    读取的二进制文件
    """
    assert fo.is_exist(file_path), f'{file_path} 文件不存在!'
    obj = joblib.load(file_path)

    return obj


def parallel(args, func, njobs, backend=0, verbose=1):
    """parallel方法用于并行计算

    Parameters
    ----------
    args : list
        参数列表
    func : funcion
        对象函数
    njobs : int
        并行数
    backend : int
        并行任务种类
        0: loky 默认
        1: multiprocessing 多进程，鲁棒性不如loky
        2: threading 多线程, 当释放GIL效率高
    verbose : int
        是否输出进度条
    Returns
    ----------
    """
    # 并行种类
    backend_dict = {0: 'loky', 1: 'multiprocessing', 2: 'threading'}
    backend = backend_dict[backend]
    if verbose == 0:
        ret = joblib.Parallel(
            n_jobs=njobs, backend=backend, verbose=0)(
            joblib.delayed(func)(arg) for arg in args)
    else:
        ret = joblib.Parallel(
            n_jobs=njobs, backend=backend, verbose=0)(
            joblib.delayed(func)(arg) for arg in tqdm(args))
    return ret


def memmap(data):
    """memmap方法用于参数共享内存的变量

    Parameters
    ----------
    data : object
        python变量
    Returns
    ----------
    内存共享的变量
    """
    tmp_folder = tempfile.mkdtemp()
    tmp_path = tool.path_join(tmp_folder, 'joblib.mmap')
    if fo.is_exist(tmp_path):  # 若存在则删除
        fo.del_file(tmp_path)
    _ = joblib.dump(data, tmp_path)
    memmap_data = joblib.load(tmp_path, mmap_mode='r+')
    return memmap_data


def load_memmap(path):
    """load_memmap方法用于读取共享数据

    Parameters
    ----------
    path : str
        文件路径

    Returns
    ----------
    """
    memmap_data = joblib.load(path, mmap_mode='r+')
    return memmap_data



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     static_graph
   Description :
   Author :       23mofang
   date：          2020/3/14
-------------------------------------------------
   Change Activity:
                   2020/3/14:
-------------------------------------------------
"""
__author__ = 'Asdil'
import matplotlib.pyplot as plt
import seaborn as sns
sns.set(style='darkgrid')


def scatter(df, x, y, label=None, size=10, shape=True, xy_limit=None, title=None,
            figsize=(16, 9), dpi=100, plot_dense=0, save_path=None):
    """scatter方法用于画散点图

    Parameters
    ----------
    df : pandas.DataFrame
        pandas DataFrame 对象
    x: str
        x数据列名
    y: str
        y数据列名
    label: str or None
        类标签列名
    size: int
        散点大小
    shape: bool
        不同类不同图形
    xy_limit: list
        xy轴坐标范围, example = [xmin, xmax, ymin, ymax]
    title: str or None
        标题
    figsize : tuple
        图形长宽比默认(16:9)
    plot_dense: int
        画出密度分布 0不画 1画密度分布圈图, 2画密度分布bar图
    dpi: int
        分辨率默认100
    save_path: str
        保存路径
    Returns
    ----------
    """
    # 设置长宽分辨率，坐标轴范围
    plt.figure(figsize=figsize, dpi=dpi)
    if xy_limit:
        plt.axis(xy_limit)
    legend = 'full' if label else None
    shape = label if shape else None
    cmap = sns.cubehelix_palette(dark=.3, light=.8, as_cmap=True)
    ax = sns.scatterplot(x=x,
                         y=y,
                         s=size,
                         hue=label,
                         style=shape,
                         legend=legend,
                         palette="Set2",
                         data=df)
    if plot_dense == 1:
        sns.kdeplot(df[x], df[y], ax=ax)
    elif plot_dense == 2:
        sns.kdeplot(df[x], df[y], ax=ax)
        sns.rugplot(df[x], color="g", ax=ax)
        sns.rugplot(df[y], vertical=True, color="r", ax=ax)

    if legend:
        plt.legend(loc='best', bbox_to_anchor=(1.094, 1.01), fancybox=True, shadow=True)
    if title:
        plt.title(f'{title}', fontsize=20)
    if save_path:
        plt.savefig(f'{save_path}', dpi=dpi)


def hist(df=None, col=None, x=None, bins=None, xticks=None, yticks=None,
         save_path=None, title=None, rotation=60, dpi=100, figsize=(16, 9), kde=True, rug=True):
    """hist方法用于画单变量柱状图

    Parameters
    ----------
    df : pd.DataFrame or None
        pandas DataFrame数据
    col: str or None
        pd.DataFrame 列名
    x: array like or None
        列表类型数据
    bins: int or None
        分箱大小
    xticks: array like or None
        x轴坐标范围
    yticks: array like or None
        y轴坐标范围
    save_path: str or None
        保存路径
    title: str or None
        标题
    rotation: int
        坐标旋转角度
    dpi: int
        保存图片分辨率
    figsize: tuple
        画布大小
    kde: bool
        是否要密度分布曲线
    rug: bool
        是否要柱状图
    Returns
    ----------
    """
    plt.figure(figsize=figsize)
    if xticks:
        plt.xticks(xticks)
        plt.xticks(rotation=rotation)
    if yticks:
        plt.yticks(yticks)
        plt.yticks(rotation=rotation)
    if df is not None:
        x = df[col]
    sns.distplot(x, kde=kde, rug=rug, bins=bins, color='g',
                 kde_kws={"color": "k", "lw": 3, "label": "KDE"},
                 hist_kws={"edgecolor": 'black'})
    plt.xlabel('X', fontdict={'family': 'Times New Roman', 'size': 24})
    plt.ylabel('Ratio', fontdict={'family': 'Times New Roman', 'size': 24})

    if title:
        plt.title(title, fontsize=24)
    if save_path:
        plt.savefig(save_path, dpi=dpi)  # 指定分辨率



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     aplot
   Description :
   Author :        Asdil
   date：          2020/2/27
-------------------------------------------------
   Change Activity:
                   2020/2/27:
-------------------------------------------------
"""
__author__ = 'Asdil'
import pandas as pd
from pyecharts.charts import Bar
from pyecharts import options as opts
from pyecharts.globals import ThemeType
from snapshot_selenium import snapshot
from pyecharts.render import make_snapshot
import plotly
import plotly.express as px


# 统计单个列的数据
def bin_data(df, col, bins):
    """bin_data方法用于统计单列数据,
        如果是标签数据直接统计个数,
        如果是数值型数据分箱后统计

    Parameters
    ----------
    df : pandas DataFrame
    col : str
        需要统计的列名
    bins : int
        分箱间隔

    Returns
    ----------
    data : pandas.core.series.Series
    """
    if bins:
        return pd.value_counts(pd.cut(df[col], bins))
    return df[col].value_counts()


# 获取其它参数
def get_kwargs(kwargs):
    """get_kwargs方法用于其它参数,
    保存成字典,简化代码

    Parameters
    ----------
    kwargs : dict
        一些其它的画图参数

    Returns
    ----------
    'target': str or bool
        分类类标签列名
    'bins': int or bool
        分箱间隔
    'overlap': book
        不同target同一col是否重叠
    'reversal': bool
        是否旋转90度
    'title': str,
        标题
    'rotate': int
        x轴标签旋转角度
    'gap': int
        x轴同一刻度不同bar的间隔
    'theme': int
        主题索引1-14
    """
    theme = {0: 'Theme-default',
             1: ThemeType.LIGHT,
             2: ThemeType.DARK,
             3: ThemeType.CHALK,
             4: ThemeType.ESSOS,
             5: ThemeType.INFOGRAPHIC,
             6: ThemeType.MACARONS,
             7: ThemeType.PURPLE_PASSION,
             8: ThemeType.ROMA,
             9: ThemeType.ROMANTIC,
             10: ThemeType.SHINE,
             11: ThemeType.VINTAGE,
             12: ThemeType.WALDEN,
             13: ThemeType.WESTEROS,
             14: ThemeType.WONDERLAND,
             }
    params = {'target': False,
              'bins': False,
              'overlap': False,
              'reversal': False,
              'title': None,
              'rotate': 0,
              'gap': 0,
              'theme': 0,
              'save_path': False
              }
    # 先设定默认值，如果传入参数改变则改变默认值
    for key in params:
        if key in kwargs:
            params[key] = kwargs[key]
    params['theme'] = theme[params['theme']]
    return params


def get_y_axis(df, col, echart, **kwargs):
    """get_y_axis方法用于获取y轴数据

    Parameters
    ----------
    df :  pandas DataFrame
    col : str
        需要统计的列名
    echart : pyechart function
        pyechart的实例，可以使柱状图折线图
    kwargs : dict
        一些其它的画图参数

    Returns
    ----------
    data : pandas.core.series.Series
        需要data数据的index作为x轴坐标
    """
    if kwargs['target']:  # 若结合分类标签统计col列
        labels = set(df[kwargs['target']].unique())  # 获取分类标签集合
        for label in labels:
            df_tmp = df[df[kwargs['target']] == label]
            data = bin_data(df_tmp, col, kwargs['bins'])
            y_values = data.values.tolist()  # y轴的值
            if kwargs['overlap']:  # 如果需要同一col不同类标签重叠
                echart.add_yaxis(
                    f"{col} {kwargs['target']}={label}",
                    y_values,
                    stack="stack1",
                    gap=f"{kwargs['gap']}%")
            else:
                echart.add_yaxis(
                    f"{col} {kwargs['target']}={label}",
                    y_values,
                    gap=f"{kwargs['gap']}%")
    else:  # 单纯统计col列
        data = bin_data(df, col, kwargs['bins'])
        y_values = data.values.tolist()  # y轴的值
        echart.add_yaxis(f"{col}", y_values)
    return data


def get_x_axis(data, echart):
    """get_x_axis方法用于

    Parameters
    ----------
    data : pandas.core.series.Series
        需要data数据的index作为x轴坐标
    echart : pyechart function
        pyechart的实例，可以使柱状图折线图

    Returns
    ----------
    """
    # 添加x轴数据
    x_labels = pd.DataFrame(data)
    x_labels = x_labels.index.astype(str).tolist()
    echart.add_xaxis(x_labels)


def bar(df, col, **kwargs):
    """bar方法用于画柱状图

    Parameters
    ----------
    df :  pandas DataFrame
    col : str
        需要统计的列名
    kwargs : dict
        一些其它的画图参数
    Returns
    ----------
    """
    kwargs = get_kwargs(kwargs)  # 获取所有默认参数
    bar = Bar(init_opts=opts.InitOpts(theme=kwargs['theme']))
    # 添加y轴数据
    data = get_y_axis(df, col, bar, **kwargs)
    # 添加x轴数据
    get_x_axis(data, bar)

    # 全局设置
    bar.set_global_opts(
        title_opts=opts.TitleOpts(
            title=kwargs['title']), xaxis_opts=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(
                rotate=kwargs['rotate'])), )
    if kwargs['reversal']:  # 旋转90度
        bar.reversal_axis()
        bar.set_series_opts(label_opts=opts.LabelOpts(position="right"))
    if kwargs['save_path']:  # 保存
        bar.render(f"{kwargs['save_path']}.html")
        make_snapshot(snapshot, bar.render(), f"{kwargs['save_path']}.png")

    return bar.render_notebook()  # 直接在jupyter上显示


def scatter(df, x, y, label=None, symbol=None, mark_size=9, width=1000, height=600, title=None,
            is_show=True, save_path=None, xyrange=None):
    """scatter方法用于画动态散点图

    Parameters
    ----------
    df : pd.DataFrame
    x: str
        df中, x轴列名
    y: str
        df中, y轴列名
    label: str or None
        df中标签名称
    symbol: str or None
        df中散点形状
    mark_size: int
        散点大小
    width: int
        画布长度
    height: int
        画布高度
    title: str or None
        标题
    save_path: str or None
        保存路径必须是html
    is_show: bool
        是否显示
    xyrange: list
        固定x，y轴范围eg: [xmin, xmax, ymin, ymax]

    Returns
    ----------
    """
    fig = px.scatter(df, x=x, y=y, color=label, symbol=symbol, width=width, height=height)
    fig.update_traces(marker=dict(size=mark_size, line=dict(width=1, color='DarkSlateGrey')),
                      selector=dict(mode='markers'))
    if xyrange:
        xmin, xmax, ymin, ymax = xyrange
    else:
        xmax = df[x].max() + df[x].max() * 0.1
        xmin = df[x].min() - df[x].min() * 0.1
        ymax = df[y].max() + df[y].max() * 0.1
        ymin = df[y].min() - df[y].min() * 0.1
    fig.update_layout(template='seaborn', title=title,
                      xaxis=dict(range=[xmin, xmax]),
                      yaxis=dict(range=[ymin, ymax]))
    if save_path:
        plotly.offline.plot(fig, filename=save_path, auto_open=False)
    if is_show:
        fig.show()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aftool']

package_data = \
{'': ['*']}

install_requires = \
['DBUtils>=1.3,<2.0',
 'PyYAML>=5.3.1,<6.0.0',
 'joblib>=0.14.1,<0.15.0',
 'loguru>=0.5.1,<0.6.0',
 'matplotlib>=3.2.0,<4.0.0',
 'numpy>=1.18.0,<2.0.0',
 'pandas>=0.25.3,<0.26.0',
 'pip>=20.0.2,<21.0.0',
 'plotly>=4.9.0,<5.0.0',
 'psutil>=5.6,<6.0',
 'pycryptodome>=3.9.8,<4.0.0',
 'pyecharts>=1.6.2,<2.0.0',
 'pymysql>=0.10.0,<0.11.0',
 'pytest>=5.3.5,<6.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'seaborn>=0.10.0,<0.11.0',
 'snapshot_selenium>=0.0.2,<0.0.3',
 'tqdm>=4.41,<5.0']

setup_kwargs = {
    'name': 'aftool',
    'version': '0.2.5',
    'description': "Asdil Fibrizo's tool",
    'long_description': "# aftool \n[![](https://travis-ci.com/Asdil/aftool.svg?branch=master)](https://travis-ci.com/Asdil/aftool)\n\nAsdil's tool\n",
    'author': 'Asdil Fibrizo',
    'author_email': 'jpl4job@126.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Asdil/aftool',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
"""
-------------------------------------------------
   File Name：     cache
   Description :
   Author :       艾登科技 Asdil
   date：          2020/7/9
-------------------------------------------------
   Change Activity:
                   2020/7/9:
-------------------------------------------------
"""
__author__ = 'Asdil'
import sys


class Cache:
    """
    Cache类用于懒惰加载数据, 这是全局的
    """
    def get(self, attr):
        """get方法用于取数据(类似字典)

        Parameters
        ----------
        attr : str
            需要取出的字段名
        Returns
        ----------
        """
        if attr in self.__dict__.keys():
            return self.__getattribute__(attr)
        else:
            return None

    def set(self, key, value):
        """set方法用于存储数据

        Parameters
        ----------
        key : str
            关键字
        value : anything
            需要存储的数据,任意类型
        Returns
        ----------
        """
        self.__setattr__(key, value)

    def upset(self, key, value):
        """upset方法用于插入或者更新存储数据

        Parameters
        ----------
        key : str
            关键字
        value : anything
            需要存储的数据,任意类型
        Returns
        ----------
        """
        if key in self.__dict__.keys():
            self.__dict__[key] = value
        else:
            self.__setattr__(key, value)


sys.modules[__name__] = Cache()  # 加入系统识别
